@@ -1,7 +1,118 @@
 """Models instantiated by the voyages api."""
 from dataclasses import dataclass
 from datetime import datetime
-from typing import Optional, Tuple
+from typing import Optional, Tuple, Iterable
+from .._internals import contains_caseless
+
+
+@dataclass(frozen=True, eq=False)
+class Vessel:
+    """Vessels.
+
+    Attributes:
+        imo: The vessel imo.
+        vessel_name: The vessel name.
+    """
+
+    imo: int
+    vessel_name: str
+
+
+@dataclass(eq=False)
+class VesselFilter:
+    """A filter used to find specific vessels.
+
+    Attributes:
+        name_like: Used to find vessel by name. When specified, vessel
+            name whose names partially match (contain) the attribute's value
+            will be returned. Matching is case-insensitive.
+    """
+
+    name_like: Optional[str] = None
+
+    def _apply(self, vessels: Iterable[Vessel]) -> Iterable[Vessel]:
+        return filter(self.__does_class_match, vessels)
+
+    def __does_class_match(self, vessel: Vessel) -> bool:
+        return not self.name_like or contains_caseless(
+            self.name_like, vessel.vessel_name
+        )
+
+
+@dataclass(frozen=True, eq=False)
+class VesselClass:
+    """A group of vessels of similar characteristics, i.e. Aframax, Panamax, etc.
+
+    Attributes:
+        vessel_class_id: The vessel class ID.
+        vessel_class_name: The vessel class name.
+        vessel_type_id: The vessel type ID.
+        vessel_type: The vessel type
+    """
+
+    vessel_class_id: int
+    vessel_class_name: str
+    vessel_type_id: int
+    vessel_type: str
+
+
+@dataclass(eq=False)
+class VesselClassFilter:
+    """A filter used to find specific vessel classes.
+
+    Attributes:
+        name_like: Used to find vessel classes by name. When specified, vessel
+            classes whose names partially match (contain) the attribute's value
+            will be returned. Matching is case-insensitive.
+    """
+
+    name_like: Optional[str] = None
+
+    def _apply(
+        self, vessel_classes: Iterable[VesselClass]
+    ) -> Iterable[VesselClass]:
+        return filter(self.__does_class_match, vessel_classes)
+
+    def __does_class_match(self, vessel_class: VesselClass) -> bool:
+        return not self.name_like or contains_caseless(
+            self.name_like, vessel_class.vessel_class_name
+        )
+
+
+@dataclass(frozen=True, eq=False)
+class VesselType:
+    """Type of vessel used for transport.
+
+    Attributes:
+        vessel_type_id: The vessel type ID.
+        vessel_type: The vessel type name.
+    """
+
+    vessel_type_id: int
+    vessel_type: str
+
+
+@dataclass(eq=False)
+class VesselTypeFilter:
+    """A filter used to find specific vessel types.
+
+    Attributes:
+        name_like: Used to find vessel types by name. When specified, vessel
+            types whose names partially match (contain) the attribute's value
+            will be returned. Matching is case-insensitive.
+    """
+
+    name_like: Optional[str] = None
+
+    def _apply(
+        self, vessel_types: Iterable[VesselType]
+    ) -> Iterable[VesselType]:
+        return filter(self.__does_type_match, vessel_types)
+
+    def __does_type_match(self, vessel_type: VesselType) -> bool:
+        return not self.name_like or contains_caseless(
+            self.name_like, vessel_type.vessel_type
+        )
 
 
 @dataclass(frozen=True)
@@ -41,6 +152,8 @@ class VoyageEventDetail:
             This is the timestamp of the final AIS point the vessel is tracked
             within a jetty or that is captured performing a ship-to-ship
             operation.
+        sts_id: String, A unique identifier assigned to each ship-to-ship
+            event. Will be identical for both participating vessels.
         geo_asset_id: Numeric ID corresponding to the geo asset in which the
             event took place. Geo assets represent maritime facilities such as
             terminals, anchorages and lightering zones. Multiple geo assets are
@@ -68,6 +181,7 @@ class VoyageEventDetail:
         other_vessel_name: String, Containing the name of the second vessel in
             case of ship-to-ship operation.
     """
+
     id: Optional[str] = None
     event_id: Optional[str] = None
     event_detail_type: Optional[str] = None
@@ -75,6 +189,7 @@ class VoyageEventDetail:
     sailing_date: Optional[datetime] = None
     start_time_of_operation: Optional[datetime] = None
     end_time_of_operation: Optional[datetime] = None
+    sts_id: Optional[str] = None
     geo_asset_id: Optional[int] = None
     geo_asset_name: Optional[str] = None
     latitude: Optional[float] = None
@@ -235,14 +350,31 @@ class Voyage:
         vessel_status_id: Numeric ID that takes the following values  1 ->
             Voyage, 2 -> Breaking, 3 -> Domestic Trade, 4-> FPSO, 5-> FPSO
             Conversion, 6-> Inactive, 7-> Storage Vessel, 9-> Conversion.
+        deadweight: Numeric, measured in tonnes [t], often shortened as DWT,
+            denotes the total carrying capacity of the vessel including cargo,
+            ballast water, stores, provisions, crew and so on.
         commercial_operator_id: Numeric ID corresponding to the maritime
             company that manages the vessel commercially.
         deleted: Boolean. This will be true if the voyage has been deleted from
             our database.
         events: The events that took place during the voyage.
         id: String. Uniquely identifies the voyage.
+        horizon_id: Numeric ID that takes the following values Unknown (0),
+            Historic (1), Historical (1), Current (2), Future (3)
+        horizon: String. It can take "Historic", "Current" or "Future" values,
+            depending on whether the voyage event is in the past (StartDate and
+            EndDate both in the past), is current (StartDate in the past and
+            EndDate in the future) or future (both StartDate and EndDate in the
+            future). Note: the notions of "past", "current" and "future" are
+            not derived by the current date, but by the comparison between the
+            voyage dates and the latest received AIS for that specific vessel.
+        latest_received_ais: Date, format YYYY-MM-DD HH:MM:SS. The most recent
+            AIS update for the vessel. It is used to define the horizon of a
+            voyage and its events.
         vessel_name: The vessel name corresponding to that IMO at the time of
             that voyage.
+        pit_vessel_name: String, The vessel name corresponding to that IMO at
+            the time of that voyage (if different than VesselName)
         vessel_type: Description of the type of the vessel, based on the
             carried cargo. Main categories are Tankers, Dry (bulk carriers),
             Containers, LNG and LPG.
@@ -264,6 +396,10 @@ class Voyage:
             and most common status is the "Voyage" one, the one in which the
             vessel continuously sails and performs operations. The other
             statuses are used for specific purposes different than voyage.
+        deadweight: Numeric, measured in tonnes [t], often shortened as DWT,
+            denotes the total carrying capacity of the vessel including cargo,
+            ballast water, stores, provisions, crew and so on.
+        year_built: Numeric, year format, the year the vessel was built.
         commercial_operator: Name of the maritime company that manages the
             vessel commercially.
         start_date: Date, format YYYY-MM-DD HH:MM:SS. The beginning of the
@@ -310,6 +446,12 @@ class Voyage:
             info are considered more accurate and reliable, whenever available.
         quantity: Numeric, measured in kilotonnes [kt]. It is the cargo
             quantity reported in at least one of the market reports.
+        quantity_source: String, it specifies the source of Quantity. If market
+            reports are available this field takes value "MarketInfo".
+            If no market reports are available
+            for this voyage, the quantity is estimated.
+            Market info is considered more accurate and reliable,
+            whenever available.
         laycan_from: Date, format YYYY-MM-DD indicates the earliest reported
             Laycan From (latest day of cancellation) across all fixtures.
         laycan_to: Date, format YYYY-MM-DD indicates the latest reported Laycan
@@ -340,12 +482,35 @@ class Voyage:
             port of the current voyage. It is computed based on AIS data.
             It includes the whole period between the two port calls and non
             operational stops as well. Accuracy depends on AIS coverage.
+        predicted_ballast_distance: Numeric, Computed distance of the ballast
+            leg based on our distance model, in nautical miles. For current
+            voyage, when vessel is ballast, it is the remaining distance
+            between the vessel position and the first load port. For
+            historical legs PredictedBallastDistance is empty.
         laden_distance: Numeric. Travelled distance in nautical miles between
             the first load port and the last discharge port of the same voyage.
             It is computed based on AIS data. Accuracy depends on AIS coverage.
+        predicted_laden_distance: Numeric, Computed distance of the laden leg
+            based on our distance model, in nautical miles. For current voyage,
+            when vessel is laden, it is the remaining distance between the
+            vessel position and the last discharge port. For historical legs
+            PredictedLadenDistance is empty.
+        suez_crossing: String, indicates whether the vessel crossed the Suez
+            canal during the voyage. Depending on the leg, it can take
+            "Laden", "Ballast" or "Both" as values.
+        panama_crossing: String, indicates whether the vessel crossed the
+            Panama canal during the voyage. Depending on the leg, it can take
+            "Laden", "Ballast" or "Both" as values.
+        canakkale_crossing: String, indicates whether the vessel crossed the
+            Canakkale strait during the voyage. Depending on the leg, it can
+            take "Laden", "Ballast" or "Both" as values.
+        bosporus_crossing: String, indicates whether the vessel crossed the
+            Bosporus strait during the voyage. Depending on the leg, it can
+            take "Laden", "Ballast" or "Both" as values.
     """
-    imo: int
-    voyage_number: int
+
+    imo: Optional[int] = None
+    voyage_number: Optional[int] = None
     vessel_type_id: Optional[int] = None
     vessel_class_id: Optional[int] = None
     vessel_status_id: Optional[int] = None
@@ -353,12 +518,18 @@ class Voyage:
     deleted: Optional[bool] = False
     events: Optional[Tuple[VoyageEvent, ...]] = None
     id: Optional[str] = None
+    horizon_id: Optional[int] = None
+    horizon: Optional[str] = None
+    latest_received_ais: Optional[datetime] = None
     vessel_name: Optional[str] = None
+    pit_vessel_name: Optional[str] = None
     vessel_type: Optional[str] = None
     vessel_class: Optional[str] = None
     trade: Optional[str] = None
     trade_id: Optional[int] = None
     vessel_status: Optional[str] = None
+    deadweight: Optional[int] = None
+    year_built: Optional[int] = None
     commercial_operator: Optional[str] = None
     start_date: Optional[datetime] = None
     first_load_arrival_date: Optional[datetime] = None
@@ -375,6 +546,7 @@ class Voyage:
     cargo_group: Optional[str] = None
     cargo_type_source: Optional[str] = None
     quantity: Optional[float] = None
+    quantity_source: Optional[str] = None
     laycan_from: Optional[datetime] = None
     laycan_to: Optional[datetime] = None
     fixture_status_id: Optional[int] = None
@@ -385,7 +557,162 @@ class Voyage:
     is_implied_by_ais: Optional[bool] = None
     has_manual_entries: Optional[bool] = None
     ballast_distance: Optional[float] = None
+    predicted_ballast_distance: Optional[float] = None
     laden_distance: Optional[float] = None
+    predicted_laden_distance: Optional[float] = None
+    suez_crossing: Optional[str] = None
+    panama_crossing: Optional[str] = None
+    canakkale_crossing: Optional[str] = None
+    bosporus_crossing: Optional[str] = None
+
+
+@dataclass(frozen=True)
+class VoyageCondensed(Voyage):
+    """Contains information about a single voyage of a vessel.
+
+    Attributes:
+            starting_port_name: String, name of the port in where the voyage
+                    started from.
+            starting_port_id: Numeric ID corresponding to the port where the
+                    voyage started from. The start of a voyage is set as the
+                    end of the previous voyage (if existing) or the first
+                    received AIS (for a new building). Voyages are
+                    consecutive and with no breaks in between, therefore a
+                    vessel is always in a voyage.
+            starting_country_id: Numeric ID corresponding to the country
+                    where the voyage started from.
+            starting_country_name: String, name of the country where the
+                    voyage started from.
+            starting_area_id_level0: Numeric ID corresponding to the level 0
+                    area where the voyage started from. Level 0 areas offer a
+                    detailed breakdown of the globe to the areas of maritime
+                    interest.
+            starting_area_name_level0: String, name of the area where the
+                    voyage started from. Level 0 areas offer a detailed
+                    breakdown of the globe to the areas of maritime interest.
+                    Examples of level 0 areas include "Arabian Gulf",
+                    "US Gulf" and "East Mediterranean".
+            first_load_port_name: String, name of the port where the vessel
+                    performed the first loading port call of a voyage.
+            first_load_port_id: Numeric ID corresponding to the port where
+                    the vessel performed the first loading port call of a
+                    voyage.
+            first_load_arrival_date: Date, format YYYY-MM-DD HH:MM:SS. The
+                    beginning of the first loading port call, including
+                    waiting time. The arrival date of a port call is
+                    calculated based on the first AIS point within the
+                    event. If the vessel waits in an anchorage area for days,
+                    this waiting time is captured as the difference between
+                    the FirstLoadArrivalDate and the
+                    FirstLoadStartTimeOfOperation.
+            first_load_start_time_of_operation: Date, format
+                    YYYY-MM-DD HH:MM:SS. Timestamp indicating the beginning of
+                    the first loading operation. This is the timestamp of the
+                    first AIS point received when the vessel is within a jetty
+                    or while performing a ship-to-ship operation.
+            first_load_sailing_date: Date, format YYYY-MM-DD HH:MM:SS. The end
+                    of the first loading port call. The sailing date of an
+                    event is calculated based on the last AIS point within the
+                    event. In the case of missing AIS data, the sailing date
+                    is derived based on the next reported location of the
+                    vessel after the event and the time without reported AIS
+                    information.
+            first_load_country_id: Numeric ID corresponding to the country
+                    where the vessel performed the first loading port call.
+            first_load_country_name: String, name of the country where the
+                    vessel performed the first loading port call.
+            first_load_area_id_level0: Numeric ID corresponding to the level 0
+                    area where the vessel performed the first loading
+                    operation of the voyage. Level 0 areas offer a detailed
+                    breakdown of the globe to the areas of maritime interest.
+            first_load_area_name_level0: String, name of the area where the
+                    vessel performed the first loading operation of the
+                    voyage. Level 0 areas offer a detailed breakdown of the
+                    globe to the areas of maritime interest. Examples of level
+                    0 areas include "Arabian Gulf", "US Gulf" and "East
+                    Mediterranean".
+            last_discharge_port_name: String, name of the port where the
+                    vessel performed the last discharging port call of a
+                    voyage.
+            last_discharge_port_id: Numeric ID corresponding to the port where
+                    the vessel performed the last discharging port call of a
+                    voyage.
+            last_discharge_arrival_date: Date, format YYYY-MM-DD HH:MM:SS. The
+                    beginning of the last discharging port call, including
+                    waiting time. The arrival date of a port call is
+                    calculated based on the first AIS point within the event.
+                    If the vessel waits in an anchorage area for days, this
+                    waiting time is captured as the difference between the
+                    LastDischargeArrivalDate and the
+                    LastDischargeStartTimeOfOperation.
+            last_discharge_start_time_of_operation: Date, format
+                    YYYY-MM-DD HH:MM:SS. Timestamp indicating the beginning of
+                    the last discharging operation. This is the timestamp of
+                    the first AIS point received when the vessel is within a
+                    jetty or while performing a ship-to-ship operation.
+            last_discharge_sailing_date: Date, format YYYY-MM-DD HH:MM:SS.
+                    Timestamp indicating the beginning of the first loading
+                    operation. This is the timestamp of the first AIS point
+                    received when the vessel is within a jetty or while
+                    performing a ship-to-ship operation.
+            last_discharge_country_id: Numeric ID corresponding to the country
+                    where the vessel performed the last discharging port call.
+            last_discharge_country_name: String, name of the country where the
+                    vessel performed the last discharging port call.
+            last_discharge_area_id_level0: Numeric ID corresponding to the
+                    level 0 area where the vessel performed the last
+                    discharging operation of the voyage. Level 0 areas offer
+                    a detailed breakdown of the globe to the areas of maritime
+                    interest.
+            last_discharge_area_name_level0: String, name of the area where
+                    the vessel performed the last discharging operation of the
+                    voyage. Level 0 areas offer a detailed breakdown of the
+                    globe to the areas of maritime interest. Examples of level
+                    0 areas include "Arabian Gulf", "US Gulf" and "East
+                    Mediterranean".
+            repairs_ind: Boolean. True if the voyage contains any port call
+                    operation performed in a dry dock or shipyard.
+            storage_ind: Boolean. True if the vessel acted as storage in the
+                    specific voyage.
+            sts_load_ind: Boolean. True if at least one loading operation has
+                    been performed by transferring cargo from another vessel
+                    through a ship-to-ship operation.
+            sts_discharge_ind: Boolean. True if at least one discharging
+                    operation has been performed by transferring cargo from
+                    another vessel through a ship-to-ship operation.
+            local_trade_ind: Boolean. True if the vessel has loaded and
+                    discharged in the same country.
+    """
+
+    starting_port_name: Optional[str] = None
+    starting_port_id: Optional[int] = None
+    starting_country_id: Optional[int] = None
+    starting_country_name: Optional[str] = None
+    starting_area_id_level0: Optional[int] = None
+    starting_area_name_level0: Optional[str] = None
+    first_load_port_name: Optional[str] = None
+    first_load_port_id: Optional[int] = None
+    first_load_arrival_date: Optional[datetime] = None
+    first_load_start_time_of_operation: Optional[datetime] = None
+    first_load_sailing_date: Optional[datetime] = None
+    first_load_country_id: Optional[int] = None
+    first_load_country_name: Optional[str] = None
+    first_load_area_id_level0: Optional[int] = None
+    first_load_area_name_level0: Optional[str] = None
+    last_discharge_port_name: Optional[str] = None
+    last_discharge_port_id: Optional[int] = None
+    last_discharge_arrival_date: Optional[datetime] = None
+    last_discharge_start_time_of_operation: Optional[datetime] = None
+    last_discharge_sailing_date: Optional[datetime] = None
+    last_discharge_country_id: Optional[int] = None
+    last_discharge_country_name: Optional[str] = None
+    last_discharge_area_id_level0: Optional[int] = None
+    last_discharge_area_name_level0: Optional[str] = None
+    repairs_ind: Optional[bool] = None
+    storage_ind: Optional[bool] = None
+    sts_load_ind: Optional[bool] = None
+    sts_discharge_ind: Optional[bool] = None
+    local_trade_ind: Optional[bool] = None
 
 
 @dataclass(frozen=True)
@@ -399,17 +726,17 @@ class VoyageGeo:
         name: Name of the geo asset. Geo assets represent maritime facilities
             such as terminals, anchorages and lightering zones. Multiple geo
             assets are grouped under the same port.
-        port_id: Numeric ID corresponding to the port. A port may be associated
-            with multiple geo assets representing different terminals and
-            anchorages within this port.
-        port_name: Name of the port. A port may be associated with multiple geo
-            assets representing different terminals and anchorages within this
-            port.
+        port_id: Numeric ID corresponding to the port. A port may be
+            associated with multiple geo assets representing different
+            terminals and anchorages within this port.
+        port_name: Name of the port. A port may be associated with multiple
+            geo assets representing different terminals and anchorages within
+            this port.
         country_id: Numeric ID corresponding to the country of the geo asset.
         country: Name of the country of the geo asset.
         area_idlevel0: Numeric ID corresponding to the level 0 area of the geo
-            asset. Level 0 areas offer a detailed breakdown of the globe to the
-            areas of maritime interest. Examples of level 0 areas include
+            asset. Level 0 areas offer a detailed breakdown of the globe to
+            the areas of maritime interest. Examples of level 0 areas include
             "Arabian Gulf", "US Gulf" and "East Mediterranean".
         area_name_level0: Name of the level 0 area of the geo asset. Level 0
             areas offer a detailed breakdown of the globe to the areas of
@@ -436,16 +763,17 @@ class VoyageGeo:
             asset. Level 3 areas the highest area grouping in our taxonomy.
             Examples of such areas are "Pacific America" or "Africa". These
             group together level 2 areas. For instance, "Pacific America"
-            groups together the level 2 areas "West Coast North America", "West
-            Coast Mexico", "West Coast Central America" and "West Coast South
-            America".
+            groups together the level 2 areas "West Coast North America",
+            "West Coast Mexico", "West Coast Central America" and "West Coast
+            South America".
         area_name_level3: Name of the level 3 area of the geo asset. Level 3
             areas the highest area grouping in our taxonomy. Examples of such
-            areas are "Pacific America" or "Africa". These group together level
-            2 areas. For instance, "Pacific America" groups together the level
-            2 areas "West Coast North America", "West Coast Mexico", "West
-            Coast Central America" and "West Coast South America".
+            areas are "Pacific America" or "Africa". These group together
+            level 2 areas. For instance, "Pacific America" groups together the
+            level 2 areas "West Coast North America", "West Coast Mexico",
+            "West Coast Central America" and "West Coast South America".
     """
+
     id: Optional[int] = None
     name: Optional[str] = None
     port_id: Optional[int] = None
@@ -472,6 +800,7 @@ class VoyagesFlat:
         event_details: List of event details that relate to the events.
         geos: Geo asset data linked in events or event details.
     """
+
     voyages: Optional[Tuple[Voyage, ...]] = None
     events: Optional[Tuple[VoyageEvent, ...]] = None
     event_details: Optional[Tuple[VoyageEventDetail, ...]] = None
@@ -490,6 +819,7 @@ class VoyagesPagedResponse:
         data: The structure that contains records retrieve for the current
             page.
     """
+
     next_page_token: Optional[str] = None
     next_request_token: Optional[str] = None
     data: Optional[Tuple[Voyage, ...]] = None
@@ -507,6 +837,25 @@ class VoyagesFlatPagedResponse:
         data: The structure that contains records retrieve for the current
             page.
     """
+
     next_page_token: Optional[str] = None
     next_request_token: Optional[str] = None
     data: Optional[VoyagesFlat] = None
+
+
+@dataclass(frozen=True)
+class VoyagesCondensedPagedResponse:
+    """Paged response for voyages in condensed format from the Voyages API.
+
+    Attributes:
+        next_page_token: String. The key that should be used as a parameter of
+            the token to retrieve the next page.
+        next_request_token: String. Populated on the last page of incremental
+            results and should be used in the next incremental update request.
+        data: The structure that contains records retrieve for the current
+            page.
+    """
+
+    next_page_token: Optional[str] = None
+    next_request_token: Optional[str] = None
+    data: Optional[Tuple[VoyageCondensed, ...]] = None
